; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Dev-Module
; Nom du fichier : DevModule - DevModule.pb
; Version du fichier : 0.9.0
; Programmation : En cours
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 03-10-2014
; Mise  jour : 01-06-2018
; Code PureBasic : V5.62
; Plateforme : Windows, Linux, MacOS X
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule DevModule
  
  Declare Initialize()
  Declare CreateWindow()
  Declare EventManager()
  
EndDeclareModule

Module DevModule
  
  IncludeFile "DevModule - Constants.pb"
  IncludeFile "DevModule - Language.pb"
  IncludeFile "DevModule - Extra.pb"
  
  Global PrefFileName.s
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< OpenDevModulePreferences (Private) <<<<<<
  
  Procedure OpenDevModulePreferences()
    
    If OpenPreferences(PrefFileName)
      
      Window::ReadPrefs("MainWin", #MainWin)
      Window::ReadPrefs("InternalStructureWin", #InternalStructureWin)
      Language::ReadPrefs("Language")
      Options::ReadPrefs("Options")
      Sticky::ReadPrefs("Sticky")
      Comments::ReadPrefs("Comments")
      Field::ReadPrefs("CodeGenerator")
      Stats::ReadPrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< CreateDevModulePreferences (Private) <<<<<<
  
  Procedure CreateDevModulePreferences()
    
    If CreatePreferences(PrefFileName)
      
      Window::WritePrefs("MainWin", #MainWin)
      Window::WritePrefs("InternalStructureWin", #InternalStructureWin)
      Language::WritePrefs("Language")
      Options::WritePrefs("Options")
      Sticky::WritePrefs("Sticky")
      Comments::WritePrefs("Comments")
      Field::WritePrefs("CodeGenerator")
      Stats::WritePrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< ShutDown (Private) <<<<<
  
  Procedure ShutDown()
    
    Window::GetCurrentPositionSize(#MainWin)
    Comments::From_GUI()
    
    CreateDevModulePreferences()
    
    For GadgetID = #CheckBox_Observator To #CheckBox_ReadBinary Step 3
      UnbindGadgetEvent(GadgetID, @MethodGadgetFollowCheckBox())
    Next
    
    UnbindGadgetEvent(#TrackBar_Tabulation_Size, @TextFollowTrackBar()) 
    UnbindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
    
    RemoveKeyboardShortcut(#MainWin, #PB_Shortcut_All)
    
    ColorPreview::Free(#PreviewColor_Editor_FrontColor)
    ColorPreview::Free(#PreviewColor_Editor_BackColor)
    
    CloseWindow(#MainWin)
    
    Language::Reset()
    Window::Reset(-1)
    Sticky::Reset()
    Options::Reset()
    CodeGenerator::Reset()
    Field::Reset(Field::#RESET_STYLE_TERMINAL)
    MatrixGenerator::Reset()
    Comments::Reset()
    AdvancedDateFormatting::Reset()
    AskPureBasicCompiler::Reset()
    Stats::Reset()
    
    PrefFileName = ""
    End
    
  EndProcedure
 
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Nom de la procdure <<<<<
  
  Procedure SaveDevModuleProjectFile()
    
    FileName.s = SaveFileRequester(Language::Message("Select project to save"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      FileName = CheckFileExtension::DoIt(FileName, "devm")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            CodeGenerator::SaveProjectFile(FileName)
          EndIf
          
        EndIf
        
      Else
        
        CodeGenerator::SaveProjectFile(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure SaveDevModuleSourceCode()
    
    FileName.s = SaveFileRequester(Language::Message("Select the Source code to save"), Options::GetProjectSavePath(), Language::Message("PureBasic SourceCode (*.pb)|*.pb"), 1)
    
    If FileName <> ""
      
      FileName = CheckFileExtension::DoIt(FileName, "pb")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            CodeGenerator::SaveSourceCode(FileName)
          EndIf
          
        EndIf
        
      Else
        
        CodeGenerator::SaveSourceCode(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure

  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Nom de la procdure <<<<<
  
  Procedure LoadDevModuleProjectFile()
    
    FileName.s = OpenFileRequester(Language::Message("Select project to open"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      CodeGenerator::LoadProjectFile(FileName)

      If GetGadgetState(#Option_Single_Instance) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable) 
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_ID) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Index) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Name) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf

      If GetGadgetState(#Option_Off_Module_Instance) = 1
        DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)    
      EndIf

      SetGadgetState(#Panel, 0)
      
    EndIf
    
  EndProcedure
  
  Procedure SaveFileDispatcher()

    Select GetGadgetState(#Panel)
    
    	Case 0, 1, 2, 3
    	  SaveDevModuleProjectFile()
    	  
    	Case 4
    	  SaveDevModuleSourceCode()
    	  
    EndSelect
    
  EndProcedure
  
  Procedure MatrixGenerator()
    
    If MatrixGenerator::Runner(#MainWin)
      
      SetGadgetState(#Option_Off_Module_Instance, 1)
      DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      
      For Index = #CheckBox_Equal To #CheckBox_Swapping Step 3
        If Index <> #CheckBox_Divide
          SetGadgetState(Index, 1)
          DisableGadget(Index + 1, 0)
          DisableGadget(Index + 2, 0)
        EndIf
      Next
      
    EndIf
    
  EndProcedure
  
  Procedure MatrixEncodedGenerator()
    
    If MatrixEncodedGenerator::Runner(#MainWin)
      CodeGenerator::Generate(1)
    EndIf
    
  EndProcedure
  
  Procedure Private_ScanInternalStructure(List FieldType.s(), EditorID)
    
    EditorMax = CountGadgetItems(EditorID)
    
    For ItemID = 0 To EditorMax
      
      Field.s = GetGadgetItemText(EditorID, ItemID)
      
      If FindString(Field, "::")
        FieldType.s = ""
      ElseIf FindString(Field, "(")
        FieldType = StringBetween::FindIt(Field, ".", "(")
      ElseIf FindString(Field, "[")
        FieldType = StringBetween::FindIt(Field, ".", "[")
      Else
        FieldType = StringField(Field, 2, ".")
      EndIf
      
      If Len(FieldType) > 1
        
        FoundAt.l = -1
        
        ForEach FieldType()
          If FieldType() = FieldType
            FoundAt = ListIndex(FieldType())
            Break
          EndIf
        Next

        If FoundAt = -1
          AddElement(FieldType())
          FieldType() = FieldType
        EndIf
        
      EndIf
      
    Next
    
  EndProcedure
  
  Procedure ScanInternalStructure()
    
    ClearGadgetItems(#ListView_Internal_Structure)
    Field::Reset(Field::#RESET_STYLE_FLUSH_INTERNAL_STRUCTURE)
    
    NewList FieldType.s()
    
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Primary)
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Secondary)
    
    ForEach FieldType()
      AddGadgetItem(#ListView_Internal_Structure, -1, FieldType()) 
      Field::AddInternalStruct(FieldType())
      FieldType() = ""
    Next
    
    FreeList(FieldType())
    
  EndProcedure
  
  Procedure InternalStructureEditor()
    
    StructName.s = GetGadgetText(#ListView_Internal_Structure)
    
    If Field::SearchInternalStruct(StructName) >= 0
      
      If Window::OpenEx(#InternalStructureWin, Language::Message("Internal Structure Editor"))
        
        TextGadget(#Text_Internal_Structure_Name, 5,5, Window::Width(#InternalStructureWin)-10, 25, Language::Message("Structure name") + " : " + StructName)
        EditorGadget(#Editor_Internal_Structure, 5, GadgetLocate::DownOf(#Text_Internal_Structure_Name, 5), Window::Width(#InternalStructureWin)-10, 325)
        
        Options::RefreshEditors(100, #Editor_Internal_Structure)
        Field::To_GUI(1)
        
        ButtonGroup::Horizontal(#Button_Internal_Structure_OK, #Button_Internal_Structure_CANCEL, 500-150*2-10, 400-35, 150, 30, 5)
        
        SetGadgetText(#Button_Internal_Structure_OK, Language::Message("OK"))
        SetGadgetText(#Button_Internal_Structure_CANCEL, Language::Message("Cancel"))
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure CloseInternalStructureEditor(Mode.b)
    
    If Mode = 1
      PastedCodeCleanUp::DoIt(#Editor_Internal_Structure)
      Field::From_GUI(1)
    EndIf
    
    Window::GetCurrentPositionSize(#InternalStructureWin)
    CloseWindow(#InternalStructureWin)
    HideWindow(#MainWin, #False)
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Initialize (Public) <<<<<
  
  Procedure Initialize()
    
    Path.s = GetPathPart(ProgramFilename())
    SetCurrentDirectory(Path)
    
    CompilerSelect #PB_Compiler_OS
        
      CompilerCase #PB_OS_Windows 
        UserPath.s = GetEnvironmentVariable("APPDATA") + #PathSep + "DevModule" + #PathSep
        
        If Left(UserPath, 2) <> "C:"
          UserPath = "C:" + #PathSep + UserPath
        EndIf
        
      CompilerCase #PB_OS_Linux
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep
        
      CompilerCase #PB_OS_MacOS
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep ; This might be wrong
        
    CompilerEndSelect
    
    BuiltPathDirectory::DoIt(UserPath)
    LanguagePath.s = GetPathPart(ProgramFilename()) + "language" + #PathSep
    
    CompilerIf #PB_Compiler_Debugger
      BuiltLanguageDefaultFiles(LanguagePath)
      
      CompilerIf #PB_Compiler_OS = #PB_OS_Linux
        LinuxDesktopFile_N_Icon()
      CompilerEndIf 
      
    CompilerEndIf 
    
    PrefFileName = UserPath + "DevModule.prefs"
    
    Window::Initialize(#MainWin, 50, 50, 885, 595, #False)
    Window::Initialize(#InternalStructureWin, 50, 50, 500, 405, #False)
    
    Language::Initialize(LanguagePath, "Franais.xml", @Language_To_GUI())
    Options::Initialize()
    AskPureBasicCompiler::Initialize()
    Comments::Initialize(#Version)
    CodeGenerator::Initialize()
    Stats::Initialize()
    Help::Initialize()
    MatrixEncodedGenerator::Initialize()
    
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ; <<<<< Chargement des icnes <<<<<
    
    UsePNGImageDecoder()
    
    Sticky::Initialize(#Button_PushPin, Path + "icons" + #PathSep,  "Push Pin 01 XP - 32.png", "Push Pin 02 XP - 32.png")
    LoadImage(#Button_New, Path + "icons" + #PathSep + "New XP - 32.png")
    LoadImage(#Button_Open, Path + "icons" + #PathSep + "Open XP - 32.png")
    LoadImage(#Button_Save, Path + "icons" + #PathSep + "Save XP - 32.png")
    LoadImage(#Button_Code, Path + "icons" + #PathSep + "Generate Code XP - 32.png")
    LoadImage(#Button_Brush, Path + "icons" + #PathSep + "Brush XP - 32.png")
    LoadImage(#Button_ClipBoard, Path + "icons" + #PathSep + "ClipBoard XP - 32.png")
    LoadImage(#Button_Language, Path + "icons" + #PathSep + "Language XP - 32.png")
    LoadImage(#Button_Quit, Path + "icons" + #PathSep + "Quit XP - 32.png")

    If FileSize(PrefFileName) = -1
      CreateDevModulePreferences()
    Else
      OpenDevModulePreferences()
    EndIf
    
    Language::LoadTableFile()
    Window::SanityCheck(#MainWin)
    Stats::Analysis()
    Help::SetPath(Language::GetFileName())
    
  EndProcedure
  
  Procedure CreateWindow()
    
    If Window::Open(#MainWin)
      
      CompilerSelect #PB_Compiler_OS
          
        CompilerCase #PB_OS_Windows 
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 25,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 805, 520)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 480)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 480)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 325, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_Zero, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_One, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Zero))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_One))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650,60)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 150, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 25, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 25, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 25, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 25, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 25, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 25, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 20, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 25, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 25, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 20, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 25, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325, 55)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 800, 545)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 805, 25)
            WebGadget(#Web_Html_Help_File, 5, 35, 805, 515, "")
            
          CloseGadgetList()

        CompilerCase #PB_OS_Linux
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35, "")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 445)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 445)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 325, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_Zero, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_One, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Zero))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_One))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 190, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 32, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 34, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 32, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 34, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 810, 535)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
          
        CompilerCase #PB_OS_MacOS ; Maybe the interface need some adjustment 
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 445)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 445)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 325, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_Zero, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_One, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Zero))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_One))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 190, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 32, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 34, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 32, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 34, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 810, 535)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
         
      CompilerEndSelect
      
      ButtonImageGroup::Vertical(#Button_PushPin, #Button_Quit, GadgetLocate::RightOf(#Panel, 5), 5, 50, 50, 2)
      
      DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      
      Help::SetComboBoxGadgetID(#ComboBox_Html_Help_Files)
      Help::SetWebGadgetID(#Web_Html_Help_File)
      
      Options::GadgetLinking(#String_Project_SavePath, 0)
      Options::GadgetLinking(#String_Editor_FontName, 1)
      Options::GadgetLinking(#String_Editor_FontSize, 2)
      Options::GadgetLinking(#PreviewColor_Editor_FrontColor, 3)
      Options::GadgetLinking(#PreviewColor_Editor_BackColor, 4)
      Options::GadgetLinking(#Editor_Module_Primary, 5)
      Options::GadgetLinking(#Editor_Module_Secondary, 6)
      Options::GadgetLinking(#Editor_Source_Code, 7)

      Comments::GadgetLinking(#String_User_Name, 0)
      Comments::GadgetLinking(#String_User_NickName, 1)
      Comments::GadgetLinking(#String_User_Email, 2)
      Comments::GadgetLinking(#Option_Use_NBSPACE_Tabulation, 3)
      Comments::GadgetLinking(#TrackBar_Tabulation_Size, 4)
      Comments::GadgetLinking(#ComboBox_Date_Format, 5)

      CodeGenerator::GadgetLinking(#CheckBox_Observator, 0)
      CodeGenerator::GadgetLinking(#CheckBox_Debugging, 1)
      CodeGenerator::GadgetLinking(#String_Module_Name, 2)
      CodeGenerator::GadgetLinking(#Option_Single_Instance, 3)
      CodeGenerator::GadgetLinking(#Editor_Module_Primary, 4)
      CodeGenerator::GadgetLinking(#Editor_Module_Secondary, 5)
      CodeGenerator::GadgetLinking(#Editor_Source_Code, 6)
      CodeGenerator::GadgetLinking(#Panel, 7)
      CodeGenerator::GadgetLinking(#Panel_Primary_Secondary, 8)
      CodeGenerator::GadgetLinking(#ListView_Internal_Structure, 9)
      CodeGenerator::GadgetLinking(#CheckBox_XML_SetGet_Node, 10)
      
      MatrixGenerator::GadgetLinking(#String_Module_Name, 0)
      MatrixGenerator::GadgetLinking(#Editor_Module_Primary, 1)
      
      Field::GadgetLinking(#ComboBox_EncodedFile_Module, 0)
      Field::GadgetLinking(#ComboBox_CryptedFile_Module, 1)
      Field::GadgetLinking(#ListView_Internal_Structure, 2)
      Field::GadgetLinking(#Editor_Internal_Structure, 3)
      
      BindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
      BindGadgetEvent(#Option_Use_ASCII_Tabulation, @TrackBarFollowOption())
      
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Alt | #PB_Shortcut_F4, #Button_Quit)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F5, #Button_Code) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F6, #Button_ClipBoard) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F7, #Button_Brush)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F1, #Shortcut_Matrix_Field_Setup)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F2, #ShortCut_Matrix_Encoded)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F3, #ShortCut_Stats_Windows)
      
      Sticky::ToggleStatus(#MainWin, #True)
      Language::To_GUI()
      Options::To_GUI()
      Field::FileModule_To_GUI()
      CodeGenerator::To_GUI()
      Comments::To_GUI()
      Help::ScanHelpFolder()
      
    Else
      
      ShutDown()
      
    EndIf

  EndProcedure
  
  Procedure EventManager()

    Repeat
      
      Select WaitWindowEvent()
          
        Case #PB_Event_CloseWindow
          
          Select EventWindow()
              
            Case #InternalStructureWin
              CloseInternalStructureEditor(0)
              
            Case #MainWin
              Shutdown()
              
          EndSelect
          
        Case #PB_Event_Menu
          
          Select EventMenu()
              
            Case #Button_ClipBoard
              SetClipboardText(GetGadgetText(#Editor_Source_Code))
              
            Case #Button_Code
              CodeGenerator::Generate()
              
            Case #Button_Brush
              CodeGenerator::PastedCodeCleanUp()
              
            Case #Shortcut_Matrix_Field_Setup
              MatrixGenerator()
              
            Case #ShortCut_Matrix_Encoded
              MatrixEncodedGenerator()
              
            Case #ShortCut_Stats_Windows
              Stats::OpenStatsWindow(#MainWin)
                
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadget()
              
            Case #Button_Internal_Structure_OK
              CloseInternalStructureEditor(1)
 
            Case #Button_Internal_Structure_CANCEL
              CloseInternalStructureEditor(0)
              
            Case #Button_Editor_FrontColor
              Options::AskNewColor(0)

            Case #Button_Editor_BackColor
              Options::AskNewColor(1)

            Case #Button_Editor_FontSelect
              Options::AskNewFont()
              
            Case #Button_Browse_Project_SavePath
              Options::AskSavePath()
              
            Case #Button_Delete_Date_Mask
              Comments::DeleteSelectedDateMask()
              
            Case #Button_Date_Format_Editor
              Comments::AskDateMaskEditor()
              
            Case #Button_PushPin
              Sticky::ToggleStatus(#MainWin)
              
            Case #Button_New
              CodeGenerator::Refresh()
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Button_Open
              LoadDevModuleProjectFile()
              
            Case #Button_Save
              SaveFileDispatcher()
              
            Case #Button_Code
              CodeGenerator::Generate()
              
            Case #Button_Brush
              CodeGenerator::PastedCodeCleanUp()
              
            Case #Button_Scan_Internal_Structure
              ScanInternalStructure()
              
            Case #ListView_Internal_Structure
              If EventType() = #PB_EventType_LeftDoubleClick
                If GetGadgetState(#ListView_Internal_Structure) >= 0
                  HideWindow(#MainWin, #True)
                  InternalStructureEditor()
                EndIf
              EndIf
              
            Case #Button_Add_EncodedFile_Module
              Field::AddFileModule(0)
              
            Case #Button_Add_CryptedFile_Module
              Field::AddFileModule(1)
              
            Case #Button_Remove_EncodedFile_Module
              Field::RemoveFileModule(0)
              
            Case #Button_Remove_CryptedFile_Module
              Field::RemoveFileModule(1)
              
            Case #Button_ClipBoard
              SetClipboardText(GetGadgetText(#Editor_Source_Code))
              
            Case #Button_Language
              Language::Requester()
              Help::SetPath(Language::GetFileName())
              Help::ScanHelpFolder()
              
            Case #Button_Quit
              Shutdown()
 
            Case #Option_Single_Instance
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_ID
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Index
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Name
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Off_Module_Instance
              DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              
            Case #ComboBox_Html_Help_Files
              Help::DirectlySelectHtmlFiles()
              
          EndSelect
          
      EndSelect
      
    ForEver
    
  EndProcedure
  
EndModule

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 beta 1 (Linux - x64)
; Folding = j5--
; EnableXP
; CompileSourceDirectory